// Author: Pete Kirkham
//
// quad store.
//
// simple delegating-reader

#ifndef _DELEGATING_READER_H_
#define _DELEGATING_READER_H_

#include "io/reader_base.h"

template <class R>
class delegating_reader
{
public:
    inline delegating_reader (R& reader) : _reader(reader), _fail(false)
    {
    }

    inline size_t  total_read ()
    {
        return _reader.total_read();
    }
    
    inline void set_failed () 
    {
        _fail = true;
    }

    inline bool    failed ()
    {
        return _fail || _reader.failed();
    }

    inline bool    ok ()
    {
        return (!_fail) && _reader.ok();
    }

    inline bool    expect (const char* expected)
    {
        int         length(strlen(expected));
        const char* actual(read(length));

        if (strcmp(expected, actual) == 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    inline bool    expect (char expected)
    {
        return expected == read();
    }

    inline const char*   read    (size_t length)
    {
        return _reader.read(length);
    }

    inline int   read    ()
    {
        return _reader.read();
    }

    inline char*   last ()
    {
        return _reader.last();
    }

    inline int   read_int    ()
    {
        int value(0);
        int digit(read());
        int sign(1);

        while (digit == ' ')
        {
            digit = read();
        }

        if (digit == '-')
        {
            sign = -1;
        }
        else if ((digit >= '0') && (digit <= '9'))
        {
            value = digit - '0';
        }

        while (((digit = read()) >= '0') && (digit <= '9'))
        {
            value = (10 * value) + (digit - '0');
        }

        return sign * value;
    }

protected:
    R&  _reader;
    bool _fail;
};

#endif // _DELEGATING_READER_H_
